@@ -6,10 +6,12 @@ use crate::arithmetic::sign::Sign;
 use crate::arithmetic::conversion::*;
 
 use crate::primes::PRIMELIST;
+use crate::primes::MERSENNE_LIST;
+
 use crate::traits::NumberTheory;
 
 
-#[derive(Debug,Clone, PartialEq)]
+#[derive(Debug,Default,Clone, PartialEq)]
  pub struct Mpz{
       pub(crate)   sign: Sign,
       pub(crate)  limbs: Vec<u64>,
@@ -58,7 +60,7 @@ use crate::traits::NumberTheory;
  }
  
  pub fn rand(len: usize,gen: fn()->u64) -> Self {
-      let interim = (0..len).map(|_| rand()).collect::<Vec<u64>>();
+      let interim = (0..len).map(|_| rng_64()).collect::<Vec<u64>>();
       Mpz::new(Sign::Positive, interim)
  }
  
@@ -118,7 +120,29 @@ use crate::traits::NumberTheory;
     }
     return true 
   }
-       
+ 
+ pub fn is_mersenne(&self) -> Option<u64> {
+    
+    let mut flag = 0u64; 
+    let mut start = 1u64; 
+    let lead = self.limbs.last().unwrap(); 
+    for i in 0..64 {
+      if *lead == start{
+         flag = i;
+         break;
+      }
+      start= (start<<1) + 1;
+    }
+    if flag == 0 {return None}
+    for i in self.limbs[..self.len()-2].iter(){
+      if *i != u64::MAX {
+      	return None
+      }
+    }
+    
+    return Some(flag + 1 + 64u64*(self.len()-1) as u64)
+ 
+ } 
     
  pub  fn set_bit(&mut self, index: usize){ //flips the bit at the index
   
@@ -152,6 +176,29 @@ use crate::traits::NumberTheory;
               return self.limbs[idx as usize].trailing_zeros() as u64 + 64u64*idx
             }
        }
+       
+ pub fn leading_zeros(&self) -> u64 {
+    let mut idx : u64 =0;
+    
+            for i in self.limbs.iter().rev(){
+                if i == &0u64{
+                   idx+=1;
+                }
+                else{
+                  break;
+                }
+            }
+        if idx == self.len() as u64{
+               return 64u64*idx
+            }
+         else{
+              return self.limbs[self.len()-1-idx as usize].leading_zeros() as u64 + 64u64*idx
+            }   
+ }  
+ 
+ pub fn bit_length(&self) -> u64 {
+ 	 64u64*self.len() as u64-self.leading_zeros()
+ }    
     
  pub fn normalize(&mut self){
       remove_lead_zeros(&mut self.limbs);
@@ -164,7 +211,7 @@ use crate::traits::NumberTheory;
  
  */
      
-   pub fn cmp(&self, other:&Self)->Ordering{
+   pub fn u_cmp(&self, other:&Self)->Ordering{
             cmp_slice(&self.limbs[..],&other.limbs[..])
    } 
  
@@ -173,6 +220,25 @@ use crate::traits::NumberTheory;
  
  */
  
+ 
+ pub fn mut_and(&mut self, other: &Self){
+        for (i,j) in self.limbs.iter_mut().zip(other.limbs.iter()){
+          *i = *i&j
+        }
+ }
+ 
+ pub fn mut_or(&mut self, other: &Self){
+        for (i,j) in self.limbs.iter_mut().zip(other.limbs.iter()){
+          *i = *i|j
+        }
+ }
+ 
+ pub fn mut_xor(&mut self, other: &Self){
+        for (i,j) in self.limbs.iter_mut().zip(other.limbs.iter()){
+          *i = *i^j
+        }
+ }
+ 
  pub fn mut_shl(&mut self, shift: usize){
     
     let mut k = self.clone();
@@ -218,6 +284,26 @@ use crate::traits::NumberTheory;
      k
  }
  
+ pub fn and(&self, other: &Self) -> Self {
+ 	let mut k = self.clone();
+ 	k.mut_and(&other);
+ 	k
+ }
+ 
+ pub fn or(&self, other: &Self) -> Self {
+ 	let mut k = self.clone();
+ 	k.mut_or(&other);
+ 	k
+ }
+ 
+ pub fn xor(&self, other: &Self) -> Self {
+ 	let mut k = self.clone();
+ 	k.mut_xor(&other);
+ 	k
+ }
+ 
+ 
+ 
   pub fn congruence_u64(&self, n: u64, c: u64) -> bool{
         mod_slice(&self.limbs[..],n) == c
   }
@@ -226,8 +312,8 @@ use crate::traits::NumberTheory;
       let mut k = n.clone();
       let mut selfie = self.clone();
 
-     if self.cmp(&n)== Ordering::Greater{
-      selfie = selfie.euclidean(&n).1;
+     if self.u_cmp(&n)== Ordering::Greater{
+      selfie = selfie.ref_euclidean(&n).1;
      }
      
       sub_slice(&mut k.limbs,&selfie.limbs);
@@ -269,11 +355,11 @@ use crate::traits::NumberTheory;
      }
      
     else{
-       if self.cmp(&other)==Ordering::Less{
+       if self.u_cmp(&other)==Ordering::Less{
              carry = sub_slice(&mut other.limbs[..],&self.limbs[..]);
              *self = other;
         }
-     else if self.cmp(&other)==Ordering::Equal{
+     else if self.u_cmp(&other)==Ordering::Equal{
            self.limbs.truncate(0);
            self.limbs.push(0);
            self.sign = Sign::Positive;
@@ -324,7 +410,7 @@ use crate::traits::NumberTheory;
   
   
   
-  pub fn euclidean(&self, other: &Self)->(Self, Self){
+  pub fn ref_euclidean(&self, other: &Self)->(Self, Self){
 
     let mut dividend = self.clone();
     
@@ -344,11 +430,11 @@ use crate::traits::NumberTheory;
     }
      
      
-     if dividend.cmp(&other)== Ordering::Equal{
+     if dividend.u_cmp(&other)== Ordering::Equal{
         return (Mpz::one(), Mpz::zero())
      }
      
-     if dividend.cmp(&other)== Ordering::Less{
+     if dividend.u_cmp(&other)== Ordering::Less{
        return (Mpz::zero(), dividend)
      }
    
@@ -374,11 +460,11 @@ use crate::traits::NumberTheory;
   */
      
  pub fn u_quadratic_residue(&self, n: &Self) -> Self{
-        self.ref_product(&self).euclidean(n).1
+        self.ref_product(&self).ref_euclidean(n).1
  }   
  
  pub fn u_mul_mod(&self, other: &Self, n: &Self) -> Self{
-      self.ref_product(other).euclidean(n).1
+      self.ref_product(other).ref_euclidean(n).1
   }
    
  pub fn u_mod_pow(&self,  y: &Self, modulo: &Self )->Self{
@@ -388,7 +474,7 @@ use crate::traits::NumberTheory;
         }
         
         let mut z = Mpz::one();
-        let mut base = self.clone().euclidean(modulo).1;
+        let mut base = self.clone().ref_euclidean(modulo).1;
         let one = Mpz::one();
         
 
@@ -398,7 +484,7 @@ use crate::traits::NumberTheory;
            return z
         }
         
-        while pow.cmp(&one) == Ordering::Greater {
+        while pow.u_cmp(&one) == Ordering::Greater {
        
        if pow.len() ==0{
          break;
@@ -484,14 +570,14 @@ use crate::traits::NumberTheory;
       }
       return false
   }
-  
+      
   pub fn sprp_check(&self, steps: usize) -> bool{
       if self.len() < 2 {return self.to_u64().unwrap().is_prime()}  // if fits into u64, reduce to 64-bit check 
     
       if self.is_fermat(){return false}
       
       for i in 0..steps{
-     let z = Mpz::rand(self.len(),rand).euclidean(&self).1;
+     let z = Mpz::rand(self.len(),rng_64).ref_euclidean(&self).1;
      
      if self.sprp(z)==false{ return false} //
    }
@@ -499,7 +585,7 @@ use crate::traits::NumberTheory;
    return true
   }
   
-    // weighted to maintain at most 2^-64 probability of failure, slower than most implementations for small numbers but faster for larger. Values greater than 2^512 receive only two checks, a strong-base 2 and a random-base check
+    // weighted to maintain at most 2^-64 probability of failure, slower than most implementations for small numbers but faster for larger. Values greater than 2^512 receive only two checks, a strong-base 2 and a random-base check. This is due to the fact that the density of pseudoprimes rapidly declines
   pub fn probable_prime(&self) -> bool{
       const CHECK_LUT : [u8;10] = [12,11,9,6,5,5,4,3,2,1];
       let mut check = 1;
@@ -510,32 +596,80 @@ use crate::traits::NumberTheory;
    let two = Mpz::from_u64(2);
 
    if self.is_even(){return false}
+   if self.is_fermat(){return false}
+   
+    match self.is_mersenne() {
+      Some(x) => {if MERSENNE_LIST.contains(&(x as u32)){return true}
+                   else if x < 57885161 {return false}
+                   else{return self.llt()} }
+      None    => (),
+    }
+    
    for i in PRIMELIST[1..].iter(){ // apparently optimal on my machine
      if self.congruence_u64(*i as u64,0){
        return false
      }
    }
-
-   if self.sprp(two) == false {return false}
-
-   return self.sprp_check(check as usize)
+   
+   if self.sprp(two)==false{return false}
+ 
+  let z = self.sprp_check(check as usize);
+     
+   return z
   
   }
-  /*
+  
   pub fn llt(&self) -> bool{
   	let mut s = Mpz::from_u64(4);
-  	for i in 0..(82589933-2){
-  	  if i%1_0==0{println!("{}",i);}
+  	let mut p = 0u64;
+  	
+  	match self.is_mersenne() {
+  	  Some(x) => p = x,
+  	  None    => return false
+  	}
+  	
+  	for i in 0..(p-2){
+  	 // if i%1000==0{println!("{}",i);}
   	  s = s.ref_product(&s);
   	  s.normalize();
   	  sub_slice(&mut s.limbs[..], &[2]);
   	  
-  	  s = s.euclidean(&self).1;
+  	  s = s.ref_euclidean(&self).1;
   	}
   	s.normalize();
   	if s == Mpz::zero(){return true}
   	return false
   }
+  
+   /*
+   fn lltmod(mut s: Self,m: &Self, p: usize) -> Self {
+    while s.bit_length() > p as u64{
+       
+       s = s.clone().and(&m).addition(s.clone().shr(p)) ;
+       
+       }
+       if &s == m{
+         return Mpz::zero();
+       }
+       return s
+   }
+  
+  pub fn llt(&self) -> bool{
+     let mut s = Mpz::from_u64(4);
+     let two = Mpz::from_u64(2);
+     let p = 521usize;
+     'outer :for i in 0..(p-2){
+      if i%10==0 {println!("{}",i)}
+      s = s.clone().ref_product(&s);
+      s.mut_subtraction(two.clone());
+      s= Mpz::lltmod(s,self,p);
+       
+     }
+      if s == Mpz::zero(){
+         return true
+       }
+       return false
+  }
   */
   // If self is a sophie prime return the safe 
   pub fn is_sophie(&self) -> Option<Self> {
@@ -565,12 +699,12 @@ use crate::traits::NumberTheory;
   }
   
    fn delta(&self, other: &Self) -> Self{
-     if self.cmp(other) == Ordering::Greater {
+     if self.u_cmp(other) == Ordering::Greater {
        let mut k = self.clone();
        sub_slice(&mut k.limbs[..],&other.limbs[..]);
        return k 
      }
-     if self.cmp(other) == Ordering::Less{
+     if self.u_cmp(other) == Ordering::Less{
        let mut k = other.clone();
         sub_slice(&mut k.limbs[..],&self.limbs[..]);
         return k
@@ -583,7 +717,7 @@ use crate::traits::NumberTheory;
   fn mod_sqr_1(&self, n: &Self) -> Self{
      let mut k = self.ref_product(&self);
      k.successor();
-     k.euclidean(n).1
+     k.ref_euclidean(n).1
   }
   
   fn gcd(&self, other: &Self)->Self{
@@ -595,7 +729,7 @@ use crate::traits::NumberTheory;
 
              t = b.clone();
 
-             b = a.euclidean(&b).1;
+             b = a.ref_euclidean(&b).1;
 
              b.normalize();
              a = t;
@@ -607,7 +741,7 @@ use crate::traits::NumberTheory;
      let mut x = Mpz::new(Sign::Positive,vec![2]); let mut y = Mpz::new(Sign::Positive,vec![2]); let mut d = Mpz::one();
   while d == Mpz::one() {
   x = x.mod_sqr_1(&self);
-  y = y.mod_sqr_1(&self).mod_sqr_1(&self).euclidean(&self).1 ;
+  y = y.mod_sqr_1(&self).mod_sqr_1(&self).ref_euclidean(&self).1 ;
   d = x.delta(&y).gcd(self);
    }
    d
@@ -629,7 +763,7 @@ use crate::traits::NumberTheory;
        let two = Mpz::new(Sign::Positive, vec![2]);
        est.limbs.push(isqrt(lead));
        for i in 0..100{
-         est = ( est.addition(self.euclidean(&est).0) ).euclidean(&two).0;
+         est = ( est.addition(self.ref_euclidean(&est).0) ).ref_euclidean(&two).0;
        }
        est
  } 
@@ -651,7 +785,7 @@ let lead = self.lead_digit();
        let root_minus =  Mpz::from_u64(y-1);
        est.limbs.push(nrt(lead,y));
        for i in 0..100{
-         est = ( (est.ref_product(&root_minus)).addition(self.euclidean(&est.pow(y-1)).0) ).euclidean(&root).0;
+         est = ( (est.ref_product(&root_minus)).addition(self.ref_euclidean(&est.pow(y-1)).0) ).ref_euclidean(&root).0;
        }
        est
 
@@ -670,17 +804,34 @@ let lead = self.lead_digit();
  }
  
  pub fn log2(&self) -> f64{
-     self.ln()*1.442695041
+     self.ln()*1.4426950408889634
  }
  
  pub fn log10(&self) -> f64{
-     self.ln()*0.434294482
+     self.ln()*0.43429448190325176
  }
  
  pub fn log(&self, log: f64) -> f64{
     self.ln() * log.ln().recip()
  }
- 
+ /*
+ pub fn pi(&self) -> f64{   // fast pi  parallelize probable prime
+    let x = self.ln();
+    //(self/x )*(1.0 + 1.0/x  + 2.0/(x.ln()*x.ln()))
+ }
+ */
+ /*
+ def mod(n,p):
+    """ Returns the value of (s**2 - 2) % (2**p -1)"""
+    Mp = (1<<p) - 1
+    while n.bit_length() > p: # For Python < 2.7 use len(bin(n)) - 2 > p
+        n = (n & Mp) + (n >> p)
+    if n == Mp:
+        return 0
+    else:
+        return n
+
+ */
  pub fn sirp(infimum: u64, supremum: u64, modulo: u64, residue: u64) -> Self{
  
  let mut sirp = Mpz::one();
