@@ -9,6 +9,28 @@ use crate::primes::PRIMELIST;
 impl NumberTheory for Mpz{
   
   
+  fn rng() -> Self {Mpz::new(Sign::Positive, vec![rng_64(), rng_64(), rng_64(), rng_64()])}
+  
+  fn euclidean(&self, other:&Self) -> (Self,Self) {
+     if self.sign==Sign::Negative || other.sign == Sign::Negative {
+        let (quo, mut rem) = self.ref_euclidean(other);
+        rem.neg();
+        return (quo,rem)
+     }
+     else if self.sign==Sign::Positive || other.sign == Sign::Negative {
+       let (mut quo, mut rem) = self.ref_euclidean(other);
+        quo.neg();
+        return (quo,rem)
+     }
+     else if self.sign==Sign::Negative || other.sign == Sign::Positive{
+       let (mut quo, mut rem) = self.ref_euclidean(other);
+        quo.neg();
+        rem.neg();
+        return (quo,rem)
+     }
+     self.ref_euclidean(other)
+  }
+  
   fn quadratic_residue(&self, n: &Self) -> Self{
   
      if n == &Mpz::zero(){
@@ -21,7 +43,7 @@ impl NumberTheory for Mpz{
        p = p.add_modinv(&n);
     }
     
-     self.ref_product(&self).euclidean(n).1
+     self.ref_product(&self).ref_euclidean(n).1
   }
   
   
@@ -42,7 +64,7 @@ impl NumberTheory for Mpz{
        q = q.add_modinv(n)
     }
   
-      p.ref_product(&q).euclidean(n).1
+      p.ref_product(&q).ref_euclidean(n).1
     }
   
   fn mod_pow(&self,  y: &Self, n: &Self )->Self{
@@ -89,7 +111,7 @@ impl NumberTheory for Mpz{
 
              t = b.clone();
 
-             b = a.euclidean(&b).1;
+             b = a.ref_euclidean(&b).1;
 
              b.normalize();
              a = t;
@@ -158,7 +180,7 @@ impl NumberTheory for Mpz{
               let mut count = 0u64;
             // println!("{:?}", n);
      'inner : loop {
-            let (inner_quo, inner_rem) = n.euclidean(&k);
+            let (inner_quo, inner_rem) = n.ref_euclidean(&k);
                           //println!("{:?}", n);
              if inner_rem != Mpz::zero(){
                 break 'inner;
@@ -216,8 +238,27 @@ impl NumberTheory for Mpz{
     numerator = numerator.ref_product(i)
    }
    
-   (self.euclidean(&denominator).0).ref_product(&numerator)
+   (self.ref_euclidean(&denominator).0).ref_product(&numerator)
  
  }
+ 
+ 
+ fn legendre(&self, p: &Self) -> i8 {
+     let mut p_minus = p.clone();
+     sub_slice(&mut p_minus.limbs[..], &[1]);
+     let pow = p_minus.ref_euclidean(&Mpz::from_u64(2)).0;
+     let k = self.mod_pow(&pow,&p);
+    if k == Mpz::one() {return 1};
+    if k == p_minus {return -1};
+    return 0
+ }
+ 
+ fn checked_legendre(&self, p: &Self) -> Option<i8> {
+      if p == &Mpz::from_u64(2) {return None}
+     match p.is_prime(){
+       true  => Some(self.legendre(p)),
+       false => None,
+     }
+ }
   
   }
