@@ -2,11 +2,17 @@ use crate::traits::NumberTheory;
 use crate::primes::PRIMELIST;
 
 use crate::fjprime64::fjprime_64;
-
+use crate::arithmetic::inlineops::*;
 
 
 impl NumberTheory for u64{
 
+ fn rng() -> Self {rng_64() }
+ 
+ fn euclidean(&self, other: &Self) -> (Self,Self) {
+   (*self/ *other, *self%*other)
+  }
+  
  fn is_prime(&self)->bool{
    
    if *self < u32::MAX as u64 { // tree down to u32 if it fits
@@ -163,12 +169,33 @@ impl NumberTheory for u64{
 
 }
 
+ fn legendre(&self, p: &Self) -> i8 {
+    let k = self.mod_pow(&((*p-1)>>1), p);
+    if k == 1{return 1};
+    if k == *p-1 {return -1};
+    return 0
+ }
+ 
+ fn checked_legendre(&self, p: &Self) -> Option<i8> {
+     if *p == 2 {return None}
+     match p.is_prime(){
+       true  => Some(self.legendre(p)),
+       false => None,
+     }
+ }
+
 }
 
 
 
 impl NumberTheory for i64{
   
+  fn rng() -> Self {rng_64() as i64 }
+   
+  fn euclidean(&self, other: &Self) -> (Self,Self) {
+    (*self/ *other, *self%*other)
+  }
+  
   fn is_prime(&self) -> bool{
     (self.abs() as u64).is_prime()
   }
@@ -218,6 +245,16 @@ impl NumberTheory for i64{
    }
      (a as u64).mod_pow( &(pow.abs() as u64), &(n.abs() as u64)) as i64
   }
+  
+  
+   fn legendre(&self, p: &Self) -> i8 {
+       (self.abs() as u64).legendre(&(p.abs() as u64))
+ }
+ 
+  fn checked_legendre(&self, p: &Self) -> Option<i8> {
+     (self.abs() as u64).checked_legendre(&(p.abs() as u64))
+ }
+ 
 }
 
   
